using System;
using System.Runtime.InteropServices;

namespace DMX2
{
	public partial class MidiEventProvider
	{
		const string ASOUND_LIB_NAME = "libasound.so.2";
		const int SND_SEQ_NONBLOCK = 1;
		const int SND_SEQ_OPEN_INPUT = 1;
		const int SND_SEQ_OPEN_OUTPUT = 2;
		const int SND_SEQ_OPEN_DUPLEX = 3;
		const int SND_SEQ_PORT_CAP_NO_EXPORT = (1 << 7);
		const int SND_SEQ_PORT_CAP_READ = (1 << 0);
		const int SND_SEQ_PORT_CAP_WRITE = (1 << 1);
		const int SND_SEQ_PORT_CAP_DUPLEX = (1 << 4);
		const int SND_SEQ_PORT_CAP_SUBS_READ=(1<<5);
		const int SND_SEQ_PORT_CAP_SUBS_WRITE =	(1<<6);
		const int SND_SEQ_USER_CLIENT = 1;
		const int SND_SEQ_PORT_SYSTEM_TIMER = 0;
		const int SND_SEQ_PORT_SYSTEM_ANNOUNCE = 1;
		const int SND_SEQ_QUERY_SUBS_READ = 0;
		const int SND_SEQ_PORT_TYPE_APPLICATION = (1<<20);

	
		enum snd_seq_event_type_t : byte {
	    	SND_SEQ_EVENT_SYSTEM = 0,
	        /** returned result status; event data type = #snd_seq_result_t */
	        SND_SEQ_EVENT_RESULT,

	        /** note on and off with duration; event data type = #snd_seq_ev_note_t */
	        SND_SEQ_EVENT_NOTE = 5,
	        /** note on; event data type = #snd_seq_ev_note_t */
	        SND_SEQ_EVENT_NOTEON,
	        /** note off; event data type = #snd_seq_ev_note_t */
	        SND_SEQ_EVENT_NOTEOFF,
	        /** key pressure change (aftertouch); event data type = #snd_seq_ev_note_t */
	        SND_SEQ_EVENT_KEYPRESS,
	        
	        /** controller; event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_CONTROLLER = 10,
	        /** program change; event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_PGMCHANGE,
	        /** channel pressure; event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_CHANPRESS,
	        /** pitchwheel; event data type = #snd_seq_ev_ctrl_t; data is from -8192 to 8191) */
	        SND_SEQ_EVENT_PITCHBEND,
	        /** 14 bit controller value; event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_CONTROL14,
	        /** 14 bit NRPN;  event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_NONREGPARAM,
	        /** 14 bit RPN; event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_REGPARAM,

	        /** SPP with LSB and MSB values; event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_SONGPOS = 20,
	        /** Song Select with song ID number; event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_SONGSEL,
	        /** midi time code quarter frame; event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_QFRAME,
	        /** SMF Time Signature event; event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_TIMESIGN,
	        /** SMF Key Signature event; event data type = #snd_seq_ev_ctrl_t */
	        SND_SEQ_EVENT_KEYSIGN,
	        /** MIDI Real Time Start message; event data type = #snd_seq_ev_queue_control_t */
	        SND_SEQ_EVENT_START = 30,
	        /** MIDI Real Time Continue message; event data type = #snd_seq_ev_queue_control_t */
	        SND_SEQ_EVENT_CONTINUE,
	        /** MIDI Real Time Stop message; event data type = #snd_seq_ev_queue_control_t */
	        SND_SEQ_EVENT_STOP,
	        /** Set tick queue position; event data type = #snd_seq_ev_queue_control_t */
	        SND_SEQ_EVENT_SETPOS_TICK,
	        /** Set real-time queue position; event data type = #snd_seq_ev_queue_control_t */
	        SND_SEQ_EVENT_SETPOS_TIME,
	        /** (SMF) Tempo event; event data type = #snd_seq_ev_queue_control_t */
	        SND_SEQ_EVENT_TEMPO,
	        /** MIDI Real Time Clock message; event data type = #snd_seq_ev_queue_control_t */
	        SND_SEQ_EVENT_CLOCK,
	        /** MIDI Real Time Tick message; event data type = #snd_seq_ev_queue_control_t */
	        SND_SEQ_EVENT_TICK,
	        /** Queue timer skew; event data type = #snd_seq_ev_queue_control_t */
	        SND_SEQ_EVENT_QUEUE_SKEW,
	        /** Sync position changed; event data type = #snd_seq_ev_queue_control_t */
	        SND_SEQ_EVENT_SYNC_POS,

	        /** Tune request; event data type = none */
	        SND_SEQ_EVENT_TUNE_REQUEST = 40,
	        /** Reset to power-on state; event data type = none */
	        SND_SEQ_EVENT_RESET,
	        /** Active sensing event; event data type = none */
	        SND_SEQ_EVENT_SENSING,

	        /** Echo-back event; event data type = any type */
	        SND_SEQ_EVENT_ECHO = 50,
	        /** OSS emulation raw event; event data type = any type */
	        SND_SEQ_EVENT_OSS,

	        /** New client has connected; event data type = #snd_seq_addr_t */
	        SND_SEQ_EVENT_CLIENT_START = 60,
	        /** Client has left the system; event data type = #snd_seq_addr_t */
	        SND_SEQ_EVENT_CLIENT_EXIT,
	        /** Client status/info has changed; event data type = #snd_seq_addr_t */
	        SND_SEQ_EVENT_CLIENT_CHANGE,
	        /** New port was created; event data type = #snd_seq_addr_t */
	        SND_SEQ_EVENT_PORT_START,
	        /** Port was deleted from system; event data type = #snd_seq_addr_t */
	        SND_SEQ_EVENT_PORT_EXIT,
	        /** Port status/info has changed; event data type = #snd_seq_addr_t */
	        SND_SEQ_EVENT_PORT_CHANGE,

	        /** Ports connected; event data type = #snd_seq_connect_t */
	        SND_SEQ_EVENT_PORT_SUBSCRIBED,
	        /** Ports disconnected; event data type = #snd_seq_connect_t */
	        SND_SEQ_EVENT_PORT_UNSUBSCRIBED,
	        /** user-defined event; event data type = any (fixed size) */
	        SND_SEQ_EVENT_USR0 = 90,
	        /** user-defined event; event data type = any (fixed size) */
	        SND_SEQ_EVENT_USR1,
	        /** user-defined event; event data type = any (fixed size) */
	        SND_SEQ_EVENT_USR2,
	        /** user-defined event; event data type = any (fixed size) */
	        SND_SEQ_EVENT_USR3,
	        /** user-defined event; event data type = any (fixed size) */
	        SND_SEQ_EVENT_USR4,
	        /** user-defined event; event data type = any (fixed size) */
	        SND_SEQ_EVENT_USR5,
	        /** user-defined event; event data type = any (fixed size) */
	        SND_SEQ_EVENT_USR6,
	        /** user-defined event; event data type = any (fixed size) */
	        SND_SEQ_EVENT_USR7,
	        /** user-defined event; event data type = any (fixed size) */
	        SND_SEQ_EVENT_USR8,
	        /** user-defined event; event data type = any (fixed size) */
	        SND_SEQ_EVENT_USR9,

	        /** system exclusive data (variable length);  event data type = #snd_seq_ev_ext_t */
	        SND_SEQ_EVENT_SYSEX = 130,
	        /** error event;  event data type = #snd_seq_ev_ext_t */
	        SND_SEQ_EVENT_BOUNCE,
	        /** reserved for user apps;  event data type = #snd_seq_ev_ext_t */
	        SND_SEQ_EVENT_USR_VAR0 = 135,
	        /** reserved for user apps; event data type = #snd_seq_ev_ext_t */
	        SND_SEQ_EVENT_USR_VAR1,
	        /** reserved for user apps; event data type = #snd_seq_ev_ext_t */
	        SND_SEQ_EVENT_USR_VAR2,
	        /** reserved for user apps; event data type = #snd_seq_ev_ext_t */
	        SND_SEQ_EVENT_USR_VAR3,
	        /** reserved for user apps; event data type = #snd_seq_ev_ext_t */
	        SND_SEQ_EVENT_USR_VAR4,

	        /** NOP; ignored in any case */
	        SND_SEQ_EVENT_NONE = 255

		}


		[StructLayout(LayoutKind.Sequential)]
		struct snd_seq_addr_t {
			public byte client;
			public byte port;
		}

		[StructLayout(LayoutKind.Sequential)]
		struct snd_seq_ev_ctrl_t {
			public byte channel;
			public byte unused1;
			public byte unused2;
			public byte unused3;
			public uint param;
			public int value;
		}

		[StructLayout(LayoutKind.Sequential)]
		struct snd_seq_ev_note_t {
			public byte channel;
			public byte note;
			public byte velocity;
			public byte off_velocity;
			public uint duration;
		}

		[StructLayout(LayoutKind.Sequential)]
		struct snd_seq_connect_t {
			public snd_seq_addr_t sender;
			public snd_seq_addr_t dest;
		}

		[StructLayout(LayoutKind.Sequential)]
		struct snd_seq_result_t {
			public int eventt;
			public int result;
		}

		[StructLayout(LayoutKind.Explicit)]
		struct snd_seq_event_t {
			[FieldOffset(0)]
			public snd_seq_event_type_t type;
			[FieldOffset(1)]
			public byte flags;
			[FieldOffset(2)]
			public byte tag;
			[FieldOffset(3)]
			public byte queue;
			[FieldOffset(4)]
			public uint time;
			[FieldOffset(12)]
			public snd_seq_addr_t source;
			[FieldOffset(14)]
			public snd_seq_addr_t dest;
			[FieldOffset(16)]
			public snd_seq_ev_ctrl_t data_ev_ctrl;
			[FieldOffset(16)]
			public snd_seq_ev_note_t data_ev_note;
			[FieldOffset(16)]
			public snd_seq_connect_t data_connect;
			[FieldOffset(16)]
			public snd_seq_result_t data_result;
		}

		[DllImport(ASOUND_LIB_NAME, CallingConvention = CallingConvention.Cdecl)]
		static extern int snd_seq_open (out IntPtr handle, string name, int streams, int mode);

		[DllImport(ASOUND_LIB_NAME, CallingConvention = CallingConvention.Cdecl)]
		static extern int snd_seq_close (IntPtr seq);

		[DllImport(ASOUND_LIB_NAME, CallingConvention = CallingConvention.Cdecl)]
		static extern int snd_seq_set_client_name (IntPtr seq, string name);

		[DllImport(ASOUND_LIB_NAME, CallingConvention = CallingConvention.Cdecl)]
		static extern int snd_seq_create_simple_port (IntPtr seq, string name, uint caps, uint type);

		[DllImport(ASOUND_LIB_NAME, CallingConvention = CallingConvention.Cdecl)]
		static extern int snd_seq_event_input (IntPtr seq, out IntPtr ev);

		[DllImport(ASOUND_LIB_NAME, CallingConvention = CallingConvention.Cdecl)]
		static extern int snd_seq_event_input_pending (IntPtr seq, int fetch_sequencer);

		[DllImport(ASOUND_LIB_NAME, CallingConvention = CallingConvention.Cdecl)]
		static extern int snd_seq_free_event(IntPtr ev);
	}
}

